<?php
// $Id: LDAPInterface.php,v 1.1.2.2 2005/06/27 15:44:29 pablobm Exp $

class LDAPInterface {
  var $instance = null;
  
  function LDAPInterface() {
    if ($this->instance !== null) {
      die('<pre><strong>LDAPInterface::LDAPInterface() error</strong>: constructor called on singleton class.</pre>');
    }
    $this->connection = null;
    $this->server = localhost;
    $this->port = 389;
    $this->secretKey = NULL;
  }
  
  function getInstance() {
    if ($this->instance === null) {
      $this->instance = new LDAPInterface();
    }
    return $this->instance;
  }

  var $connection;
  var $server;
  var $port;
  
  // Setters & getters
  function setServer($server) { $this->server = $server; }
  function getServer() { return $this->server; }
  function setPort($port) { $this->port = $port; }
  function getPort() { return $this->port; }
    
  function connect($dn = '', $pass = '') {
    $ret = FALSE;

    // If a connection already exists, it should be terminated
    $this->disconnect();
    
    if ($this->connectAndBind($dn, $pass)) {
      $ret = TRUE;
    }

    return $ret;
  }
  
  function connect_ADstyle($name = '', $base_dn = '', $pass = '') {
    global $ldap;
    
    // If a connection already exists, it should be terminated
    $this->disconnect();
    $this->establishConnection();

    ob_start();    
    $anon_bind = @ldap_bind($this->connection, LDAP_READER_USER_DN, LDAP_READER_USER_PASS);
    
    // Bind anonymously
    $res = false;
    $dn = 'cn=' . $name;
    global $ldap;
    $anon_res = @ldap_search($this->connection, variable_get('ldap_base_dn', ''), $dn);
    if ($anon_res) {
      $users = @ldap_get_entries($this->connection, $anon_res);
      
      $this->disconnect();
      $this->establishConnection();

      $user_dn = $users[0]["dn"];
      $res = @ldap_bind($this->connection, $user_dn, $pass);
    }
    else {
      $this->disconnect();
    }
    ob_end_clean();
   
    return $res;
  }
  
  function establishConnection() {
    if (!$con = ldap_connect($this->server, $this->port)) {
      watchdog('user', 'LDAP Connect failure to ' . $this->server . ':' . $this->port);
      return NULL;
    }
    $this->connection = $con;
    ldap_set_option($con, LDAP_OPT_PROTOCOL_VERSION, 3);
  }
  
  function connectAndBind($dn = '', $pass = '') {
    $this->establishConnection();

    //die('con: ' . $con . ', dn: ' . $dn . ', pass: ' . $pass . ', server: ' . $this->server . ', port: ' . $this->port);
    // We don't want anonymous connections here
    if (!$dn || !$pass || !ldap_bind($this->connection, $dn, $pass)) {
      watchdog('user', t('LDAP Bind failure for user %user. Error %errno: %error', array('%user' => $dn,'%errno' => ldap_errno($con), '%error' => ldap_error($con))));
      return NULL;
    }

    return $con;
  }
  
  
  function disconnect() {
    if ($this->connection) {
      ldap_unbind($this->connection);
      $this->connection = NULL;
    }
  }
  
  function retrieveAttributes($dn) {
    $result = ldap_read($this->connection, $dn, 'objectClass=*');
    $entries = ldap_get_entries($this->connection, $result);
    return $entries[0];
  }
  
  function retrieveAttribute($dn, $attrname) {
    $entries = $this->retrieveAttributes($dn);

    return $entries[$attrname][0];
  }
  
  function writeAttributes($dn, $attributes) {
    ldap_modify($this->connection, $dn, $attributes);
  }
}

?>